from finbourne_access.extensions.api_client_factory import SyncApiClientFactory, ApiClientFactory
from finbourne_access.extensions.configuration_loaders import (
    ConfigurationLoader,
    SecretsFileConfigurationLoader,
    EnvironmentVariablesConfigurationLoader,
    ArgsConfigurationLoader,
)


# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, constr, validator
from finbourne_access.models.role_resource_request import RoleResourceRequest
from finbourne_access.models.when_spec import WhenSpec

class RoleCreationRequest(BaseModel):
    """
    Request to create a role  # noqa: E501
    """
    code: constr(strict=True, min_length=1) = Field(..., description="The code of the role")
    description: Optional[constr(strict=True, max_length=1024, min_length=0)] = Field(None, description="The description of the role")
    resource: RoleResourceRequest = Field(...)
    when: WhenSpec = Field(...)
    __properties = ["code", "description", "resource", "when"]

    @validator('code')
    def code_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^(?=.*[a-zA-Z])[\w][\w +-]{2,100}$", value):
            raise ValueError(r"must validate the regular expression /^(?=.*[a-zA-Z])[\w][\w +-]{2,100}$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RoleCreationRequest:
        """Create an instance of RoleCreationRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of resource
        if self.resource:
            _dict['resource'] = self.resource.to_dict()
        # override the default output from pydantic by calling `to_dict()` of when
        if self.when:
            _dict['when'] = self.when.to_dict()
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RoleCreationRequest:
        """Create an instance of RoleCreationRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RoleCreationRequest.parse_obj(obj)

        _obj = RoleCreationRequest.parse_obj({
            "code": obj.get("code"),
            "description": obj.get("description"),
            "resource": RoleResourceRequest.from_dict(obj.get("resource")) if obj.get("resource") is not None else None,
            "when": WhenSpec.from_dict(obj.get("when")) if obj.get("when") is not None else None
        })
        return _obj



# coding: utf-8

# flake8: noqa
"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from finbourne_access.models.access_controlled_action import AccessControlledAction
from finbourne_access.models.access_controlled_resource import AccessControlledResource
from finbourne_access.models.action_id import ActionId
from finbourne_access.models.add_policy_collection_to_role_request import AddPolicyCollectionToRoleRequest
from finbourne_access.models.add_policy_to_role_request import AddPolicyToRoleRequest
from finbourne_access.models.add_to_policy_collection_request import AddToPolicyCollectionRequest
from finbourne_access.models.as_at_predicate_contract import AsAtPredicateContract
from finbourne_access.models.as_at_range_for_spec import AsAtRangeForSpec
from finbourne_access.models.as_at_relative import AsAtRelative
from finbourne_access.models.attached_policy_definition_response import AttachedPolicyDefinitionResponse
from finbourne_access.models.date_quality import DateQuality
from finbourne_access.models.date_unit import DateUnit
from finbourne_access.models.effective_date_has_quality import EffectiveDateHasQuality
from finbourne_access.models.effective_date_relative import EffectiveDateRelative
from finbourne_access.models.effective_range import EffectiveRange
from finbourne_access.models.entitlement_metadata import EntitlementMetadata
from finbourne_access.models.evaluation_request import EvaluationRequest
from finbourne_access.models.evaluation_response import EvaluationResponse
from finbourne_access.models.evaluation_result import EvaluationResult
from finbourne_access.models.for_spec import ForSpec
from finbourne_access.models.grant import Grant
from finbourne_access.models.how_spec import HowSpec
from finbourne_access.models.id_selector_definition import IdSelectorDefinition
from finbourne_access.models.identifier_part_schema import IdentifierPartSchema
from finbourne_access.models.if_expression import IfExpression
from finbourne_access.models.if_feature_chain_expression import IfFeatureChainExpression
from finbourne_access.models.if_identity_claim_expression import IfIdentityClaimExpression
from finbourne_access.models.if_identity_scope_expression import IfIdentityScopeExpression
from finbourne_access.models.if_request_header_expression import IfRequestHeaderExpression
from finbourne_access.models.key_value_pair_of_string_to_string import KeyValuePairOfStringToString
from finbourne_access.models.link import Link
from finbourne_access.models.lusid_problem_details import LusidProblemDetails
from finbourne_access.models.lusid_validation_problem_details import LusidValidationProblemDetails
from finbourne_access.models.match_all_selector_definition import MatchAllSelectorDefinition
from finbourne_access.models.metadata_expression import MetadataExpression
from finbourne_access.models.metadata_selector_definition import MetadataSelectorDefinition
from finbourne_access.models.non_transitive_supervisor_role_resource import NonTransitiveSupervisorRoleResource
from finbourne_access.models.operator import Operator
from finbourne_access.models.point_in_time_specification import PointInTimeSpecification
from finbourne_access.models.policy_collection_creation_request import PolicyCollectionCreationRequest
from finbourne_access.models.policy_collection_id import PolicyCollectionId
from finbourne_access.models.policy_collection_response import PolicyCollectionResponse
from finbourne_access.models.policy_collection_update_request import PolicyCollectionUpdateRequest
from finbourne_access.models.policy_creation_request import PolicyCreationRequest
from finbourne_access.models.policy_id import PolicyId
from finbourne_access.models.policy_id_role_resource import PolicyIdRoleResource
from finbourne_access.models.policy_response import PolicyResponse
from finbourne_access.models.policy_selector_definition import PolicySelectorDefinition
from finbourne_access.models.policy_type import PolicyType
from finbourne_access.models.policy_update_request import PolicyUpdateRequest
from finbourne_access.models.relative_to_date_time import RelativeToDateTime
from finbourne_access.models.remove_from_policy_collection_request import RemoveFromPolicyCollectionRequest
from finbourne_access.models.request_details import RequestDetails
from finbourne_access.models.requested_action_key import RequestedActionKey
from finbourne_access.models.resource_details import ResourceDetails
from finbourne_access.models.resource_list_of_access_controlled_resource import ResourceListOfAccessControlledResource
from finbourne_access.models.resource_list_of_policy_collection_response import ResourceListOfPolicyCollectionResponse
from finbourne_access.models.resource_list_of_policy_response import ResourceListOfPolicyResponse
from finbourne_access.models.resource_list_of_user_role_response import ResourceListOfUserRoleResponse
from finbourne_access.models.role_creation_request import RoleCreationRequest
from finbourne_access.models.role_id import RoleId
from finbourne_access.models.role_resource_request import RoleResourceRequest
from finbourne_access.models.role_response import RoleResponse
from finbourne_access.models.role_update_request import RoleUpdateRequest
from finbourne_access.models.selector_definition import SelectorDefinition
from finbourne_access.models.text_operator import TextOperator
from finbourne_access.models.user_role_creation_request import UserRoleCreationRequest
from finbourne_access.models.user_role_response import UserRoleResponse
from finbourne_access.models.user_role_update_request import UserRoleUpdateRequest
from finbourne_access.models.when_spec import WhenSpec



# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field
from finbourne_access.models.non_transitive_supervisor_role_resource import NonTransitiveSupervisorRoleResource
from finbourne_access.models.policy_id_role_resource import PolicyIdRoleResource

class RoleResourceRequest(BaseModel):
    """
    RoleResourceRequest  # noqa: E501
    """
    non_transitive_supervisor_role_resource: Optional[NonTransitiveSupervisorRoleResource] = Field(None, alias="nonTransitiveSupervisorRoleResource")
    policy_id_role_resource: Optional[PolicyIdRoleResource] = Field(None, alias="policyIdRoleResource")
    __properties = ["nonTransitiveSupervisorRoleResource", "policyIdRoleResource"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RoleResourceRequest:
        """Create an instance of RoleResourceRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of non_transitive_supervisor_role_resource
        if self.non_transitive_supervisor_role_resource:
            _dict['nonTransitiveSupervisorRoleResource'] = self.non_transitive_supervisor_role_resource.to_dict()
        # override the default output from pydantic by calling `to_dict()` of policy_id_role_resource
        if self.policy_id_role_resource:
            _dict['policyIdRoleResource'] = self.policy_id_role_resource.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RoleResourceRequest:
        """Create an instance of RoleResourceRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RoleResourceRequest.parse_obj(obj)

        _obj = RoleResourceRequest.parse_obj({
            "non_transitive_supervisor_role_resource": NonTransitiveSupervisorRoleResource.from_dict(obj.get("nonTransitiveSupervisorRoleResource")) if obj.get("nonTransitiveSupervisorRoleResource") is not None else None,
            "policy_id_role_resource": PolicyIdRoleResource.from_dict(obj.get("policyIdRoleResource")) if obj.get("policyIdRoleResource") is not None else None
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict
from pydantic import BaseModel, Field
from finbourne_access.models.as_at_predicate_contract import AsAtPredicateContract

class AsAtRangeForSpec(BaseModel):
    """
    AsAtRangeForSpec
    """
    var_from: AsAtPredicateContract = Field(..., alias="from")
    to: AsAtPredicateContract = Field(...)
    __properties = ["from", "to"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AsAtRangeForSpec:
        """Create an instance of AsAtRangeForSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of var_from
        if self.var_from:
            _dict['from'] = self.var_from.to_dict()
        # override the default output from pydantic by calling `to_dict()` of to
        if self.to:
            _dict['to'] = self.to.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AsAtRangeForSpec:
        """Create an instance of AsAtRangeForSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AsAtRangeForSpec.parse_obj(obj)

        _obj = AsAtRangeForSpec.parse_obj({
            "var_from": AsAtPredicateContract.from_dict(obj.get("from")) if obj.get("from") is not None else None,
            "to": AsAtPredicateContract.from_dict(obj.get("to")) if obj.get("to") is not None else None
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, StrictInt
from finbourne_access.models.date_unit import DateUnit
from finbourne_access.models.point_in_time_specification import PointInTimeSpecification
from finbourne_access.models.relative_to_date_time import RelativeToDateTime

class AsAtRelative(BaseModel):
    """
    AsAtRelative
    """
    var_date: Optional[PointInTimeSpecification] = Field(None, alias="date")
    adjustment: Optional[StrictInt] = None
    unit: Optional[DateUnit] = None
    relative_to_date_time: Optional[RelativeToDateTime] = Field(None, alias="relativeToDateTime")
    __properties = ["date", "adjustment", "unit", "relativeToDateTime"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AsAtRelative:
        """Create an instance of AsAtRelative from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AsAtRelative:
        """Create an instance of AsAtRelative from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AsAtRelative.parse_obj(obj)

        _obj = AsAtRelative.parse_obj({
            "var_date": obj.get("date"),
            "adjustment": obj.get("adjustment"),
            "unit": obj.get("unit"),
            "relative_to_date_time": obj.get("relativeToDateTime")
        })
        return _obj



# flake8: noqa

# import apis into api package
from finbourne_access.api.application_metadata_api import ApplicationMetadataApi
from finbourne_access.api.policies_api import PoliciesApi
from finbourne_access.api.roles_api import RolesApi
from finbourne_access.api.user_roles_api import UserRolesApi




# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class Operator(str, Enum):
    """
    Operator
    """

    """
    allowed enum values
    """
    UNDEFINED = 'Undefined'
    EQUALS = 'Equals'
    NOTEQUALS = 'NotEquals'
    GREATERTHAN = 'GreaterThan'
    GREATERTHANOREQUALTO = 'GreaterThanOrEqualTo'
    LESSTHAN = 'LessThan'
    LESSTHANOREQUALTO = 'LessThanOrEqualTo'
    IN = 'In'

    @classmethod
    def from_json(cls, json_str: str) -> Operator:
        """Create an instance of Operator from a JSON string"""
        return Operator(json.loads(json_str))



# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic import BaseModel, StrictStr, conlist
from finbourne_access.models.key_value_pair_of_string_to_string import KeyValuePairOfStringToString

class HowSpec(BaseModel):
    """
    HowSpec
    """
    type: Optional[StrictStr] = None
    parameters: Optional[conlist(KeyValuePairOfStringToString)] = None
    __properties = ["type", "parameters"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> HowSpec:
        """Create an instance of HowSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in parameters (list)
        _items = []
        if self.parameters:
            for _item in self.parameters:
                if _item:
                    _items.append(_item.to_dict())
            _dict['parameters'] = _items
        # set to None if type (nullable) is None
        # and __fields_set__ contains the field
        if self.type is None and "type" in self.__fields_set__:
            _dict['type'] = None

        # set to None if parameters (nullable) is None
        # and __fields_set__ contains the field
        if self.parameters is None and "parameters" in self.__fields_set__:
            _dict['parameters'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> HowSpec:
        """Create an instance of HowSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return HowSpec.parse_obj(obj)

        _obj = HowSpec.parse_obj({
            "type": obj.get("type"),
            "parameters": [KeyValuePairOfStringToString.from_dict(_item) for _item in obj.get("parameters")] if obj.get("parameters") is not None else None
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, constr, validator

class PolicyId(BaseModel):
    """
    PolicyId
    """
    scope: Optional[constr(strict=True, max_length=100, min_length=3)] = None
    code: constr(strict=True, max_length=100, min_length=3) = Field(...)
    __properties = ["scope", "code"]

    @validator('scope')
    def scope_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^(?=.*[a-zA-Z])[\w][\w +-]{2,100}$", value):
            raise ValueError(r"must validate the regular expression /^(?=.*[a-zA-Z])[\w][\w +-]{2,100}$/")
        return value

    @validator('code')
    def code_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^(?=.*[a-zA-Z])[\w][\w +-]{2,100}$", value):
            raise ValueError(r"must validate the regular expression /^(?=.*[a-zA-Z])[\w][\w +-]{2,100}$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PolicyId:
        """Create an instance of PolicyId from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if scope (nullable) is None
        # and __fields_set__ contains the field
        if self.scope is None and "scope" in self.__fields_set__:
            _dict['scope'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PolicyId:
        """Create an instance of PolicyId from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PolicyId.parse_obj(obj)

        _obj = PolicyId.parse_obj({
            "scope": obj.get("scope"),
            "code": obj.get("code")
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic import BaseModel, Field, conlist, constr
from finbourne_access.models.entitlement_metadata import EntitlementMetadata
from finbourne_access.models.policy_collection_id import PolicyCollectionId
from finbourne_access.models.policy_id import PolicyId

class PolicyCollectionUpdateRequest(BaseModel):
    """
    Update an existing PolicyCollection  # noqa: E501
    """
    policies: Optional[conlist(PolicyId)] = Field(None, description="The identifiers of the Policies in this collection")
    metadata: Optional[Dict[str, conlist(EntitlementMetadata)]] = Field(None, description="Any relevant metadata associated with this resource for controlling access to this resource")
    policy_collections: Optional[conlist(PolicyCollectionId)] = Field(None, alias="policyCollections", description="The identifiers of the PolicyCollections in this collection")
    description: Optional[constr(strict=True, max_length=1024, min_length=0)] = Field(None, description="A description of this policy collection")
    __properties = ["policies", "metadata", "policyCollections", "description"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PolicyCollectionUpdateRequest:
        """Create an instance of PolicyCollectionUpdateRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in policies (list)
        _items = []
        if self.policies:
            for _item in self.policies:
                if _item:
                    _items.append(_item.to_dict())
            _dict['policies'] = _items
        # override the default output from pydantic by calling `to_dict()` of each value in metadata (dict of array)
        _field_dict_of_array = {}
        if self.metadata:
            for _key in self.metadata:
                if self.metadata[_key]:
                    _field_dict_of_array[_key] = [
                        _item.to_dict() for _item in self.metadata[_key]
                    ]
            _dict['metadata'] = _field_dict_of_array
        # override the default output from pydantic by calling `to_dict()` of each item in policy_collections (list)
        _items = []
        if self.policy_collections:
            for _item in self.policy_collections:
                if _item:
                    _items.append(_item.to_dict())
            _dict['policyCollections'] = _items
        # set to None if policies (nullable) is None
        # and __fields_set__ contains the field
        if self.policies is None and "policies" in self.__fields_set__:
            _dict['policies'] = None

        # set to None if metadata (nullable) is None
        # and __fields_set__ contains the field
        if self.metadata is None and "metadata" in self.__fields_set__:
            _dict['metadata'] = None

        # set to None if policy_collections (nullable) is None
        # and __fields_set__ contains the field
        if self.policy_collections is None and "policy_collections" in self.__fields_set__:
            _dict['policyCollections'] = None

        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PolicyCollectionUpdateRequest:
        """Create an instance of PolicyCollectionUpdateRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PolicyCollectionUpdateRequest.parse_obj(obj)

        _obj = PolicyCollectionUpdateRequest.parse_obj({
            "policies": [PolicyId.from_dict(_item) for _item in obj.get("policies")] if obj.get("policies") is not None else None,
            "metadata": dict(
                (_k,
                        [EntitlementMetadata.from_dict(_item) for _item in _v]
                        if _v is not None
                        else None
                )
                for _k, _v in obj.get("metadata").items()
            ),
            "policy_collections": [PolicyCollectionId.from_dict(_item) for _item in obj.get("policyCollections")] if obj.get("policyCollections") is not None else None,
            "description": obj.get("description")
        })
        return _obj
